#lang racket
(require rackunit)
;; This is mostly a translation of Common Lisp examples
;; from http://longino.com.br/article.html?file=continuations
;; into tested Racket code

;------------------ Part 1/4 ------------------
(displayln "~~ 1/4 ~~")
(define continuation null)

(let ([x 0])
  ;; call/cc will pass the continuation to the specified function,
  ;; that in turn will store it in the variable named "continuation".
  (call/cc (λ (k) (set! continuation k)))
  
  ;; the continuation will represent the process from this point and on,
  ;; including its context with its local variables.
  
  ;; increments x and return the new value.
  (begin (set! x (add1 x)) x))
;; returned value: 1

(check-=
 (continuation) 2 0)
;; returned value: 2

(check-=
 (continuation) 3 0)
;; returned value: 3

(check-=
 (continuation) 4 0)
;; returned value: 4

;------------------ Part 2/4 ------------------
(displayln "~~ 2/4 ~~")
(set! continuation null)

(apply + 1
       (call/cc (λ (k)
                  (set! continuation k) '(1))))
;; returned value: 2

(check-=
 (continuation '(10)) 11 0)
;; returned value: 11

(check-=
 (continuation '(50)) 51 0)
;; returned value: 51

(check-=
 (continuation (list 7 13)) 21 0)
;; returned value: 21

;------------------ Part 3/4 ------------------
(displayln "~~ 3/4 ~~")
(define (find-number number list)
  (call/cc (λ (k) 
             (for ([item (in-list list)]
                   [index (in-naturals 1)]
                   #:when (= item number))
               (k index))
             null)))

(check-=
 (find-number 20 '(10 20 30 40)) 2 0)
;; returned value: 2 

(check-pred null?
            (find-number 50 '(10 20 30 40)))
;; returned value: null

;------------------ Parte 4/4 ------------------
(displayln "~~ 4/4 ~~")
(define (first-fn other-fn)
  (display 1)
  (set! other-fn (call/cc other-fn))
  (display 2)
  (set! other-fn (call/cc other-fn))
  (display 3)
  (set! other-fn (call/cc other-fn)))

(define (second-fn other-fn)
  (display #\a)
  (set! other-fn (call/cc other-fn))
  (display #\b)
  (set! other-fn (call/cc other-fn))
  (display #\c)
  (set! other-fn (call/cc other-fn)))

(first-fn second-fn)
;; The printed value will be 1a2b3c
