#lang slideshow
; Rodolfo Henrique Carvalho
; 2011/05/17

(require slideshow/code)
(require scheme/pretty)

(set-page-numbers-visible! #f)

;; Library.

(define (large-text txt)
  (text txt (current-main-font) 62))

(define (pretty-syntax sexp . cols)
  (let ((port (open-input-string (apply pretty-format (cons sexp cols)))))
    (port-count-lines! port)
    (read-syntax "<no-name>" port)))

(define-syntax code-reduce
  (syntax-rules ()
    ((_ exp) 
     (let ((t exp))
       (hb-append (code exp) (tt " => ") (code (unsyntax t)))))))

(define comma ", ")

(define (fact n)
  (cond 
    ((<= n 0) 1)
    (else (* n (fact (- n 1))))))

(define logo
  (bitmap (build-path (collection-path "icons") "plt-logo-red-shiny.png")))


;; Slides.


(slide
 (para #:align 'center 
       (large-text "Introdução ao Racket"))
 (blank 50)
 logo
 (vc-append 0
            (t "Rodolfo Carvalho")
            (it "baseado nos slides de Matthew Might")
            (blank 50)
            (colorize (bt "Dojo Rio") "red")
            (colorize (t "rodolfocarvalho.net") "blue")))


(slide
 #:title "Por que aprender Racket (ou Lisp)?"
 (para #:align 'center #:width 500
       "Um bom programador FORTRAN pode escrever FORTRAN em qualquer linguagem.")
 (blank 75)
 'next
 (para #:align 'center #:width 450 "Um bom programador Lisp pode escrever qualquer linguagem em Lisp.")
 )


(slide
 #:title "Por que aprender Racket (ou Lisp)?"
 (para #:align 'center #:width 400
       "Um bom programador C pode escrever C em qualquer linguagem.")
 (blank 75)
 (para #:align 'center #:width 450 "Um bom programador Racket pode escrever qualquer linguagem em Racket.")
 )



(slide
 #:title "O que é Racket?"
 
 (item "Lisp é λ-calculus bem feito.")
 (item "Scheme é Lisp bem feito.")
 (item "Racket é Scheme bem feito."))



(slide
 #:title "Origens do Scheme: Lisp"
 #:layout 'center
 (ht-append
  50
  (vc-append
   (scale (bitmap "jmcbw.jpg") 0.5)
   (t "John McCarthy"))
  (vl-append
   30   
   (item #:width 200 (t "Lisp criado em 1958"))
   (item #:width 200 (t "Baseado em λ-calculus"))
   (item #:width 200 (t "Baseado em processamento de lista"))
   (item #:width 200 (t "Sintaxe S-Expression"))
   (item #:width 200 (t "Código funciona como dado"))
   (item #:width 200 (t "Dado funciona como código"))
   )))


(slide
 #:title "Sintaxe básica de uma S-Expression"
 
 (tt "<s-exp> ::= <symbol>     ")
 (tt "         |  (<s-exp> ...)"))


(slide
 #:title "S-Expressions: Benefícios"
 (item "Legível")
 (item "Escrevível")
 (item "Fácil de fazer parsing"))



(slide
 #:title "Exemplo de S-Expression"
 
 (vl-append
  0
  (tt "(navios                  ")
  (tt " (navio (name Enterprise)")
  (tt "       (x    120)        ")
  (tt "       (y    150))       ")
  (tt " (navio (name Galactica) ")
  (tt "       (x    130)        ")
  (tt "       (y    42)))       ")))

(slide
 #:title "XML: S-Expressions de alto teor calórico"
 (vl-append 
  0
  (tt "<navios>")
  
  (tt " <navio>") 
  (tt "  <name>Enterprise</name>")
  (tt "  <x>120</x>")
  (tt "  <y>150</y>")
  (tt " </navio>")
  
  (tt " <navio>") 
  (tt "  <name>Galactica</name>")
  (tt "  <x>130</x>")
  (tt "  <y>42</y>")
  (tt " </navio>")
  
  (tt "</navios>")
  ))


(slide
 #:title "S-Expressions: Sintaxe universal"
 (vl-append
  0
  (tt "class Cachorro extends Animal {")
  (tt " void latir() {")
  (tt "  print(\"au-au\") ; ")
  (tt "}}"))
 'next
 'alts
 (list
  (list (vl-append
         0
         (tt "(class Cachorro extends Animal")
         (tt " (void (latir) ")
         (tt "  (print \"au-au\")))")))
  
  (list (vl-append
         0
         (tt "(class")
         (tt " (name    Cachorro)")
         (tt " (extends Animal)")
         (tt " (methods ")
         (tt "  (method ")
         (tt "   (name latir)")
         (tt "   (args ())")
         (tt "   (type (-> () void)) ")
         (tt "   (body (print \"au-au\"))))))")))))




(slide
 #:title "Scheme"
 #:layout 'center
 (ht-append
  50
  (vc-append
   (bitmap "steele.jpg")
   (t "Guy Steele"))
  (vl-append
   20
   (item #:width 200 (t "Lisp bem feito, 1978"))
   (item #:width 200 (t "Corrigiu problemas com escopo"))
   (item #:width 200 (t "Purificou a semântica formal"))
   (item #:width 200 (t "Sistema de macros higiênico"))
   )))




(slide
 #:title "Racket"
 #:layout 'center
 (ht-append
  50
  (vc-append
   (bitmap "matthew.jpg")
   (t "Matthew Flatt"))
  (vl-append
   20
   (item #:width 200 (t "Scheme bem feito, 2010"))
   (item #:width 200 (t "Muitas bibliotecas"))
   (item #:width 200 (t "Módulos, structs, etc."))
   (item #:width 200 (t "Baterias incluídas"))
   )))



(slide
 #:title "Exemplo em Racket: Fatorial"
 (code
  (define (fact n)
    (cond
      ((<= n 0)  1)      
      (else      (* n (fact (- n 1))))))))


(slide
 #:title "Programas em Racket"
 (tt "<body> ::= <def> ... <exp> ..."))

(slide
 #:title "Definições"
 (vl-append
  20
  (tt "<def> ::= (define (<id> <params>) <body>)")
  (tt "       |  (define <id> <exp>)")))

(define (id x) x)

(define (subset-scheme-slide show-core show-global show-basic show-sugar)
  (slide
   #:title "Um subconjunto das expressões de Scheme"
   (vl-append
    (tt "<exp> ::= <var>")
    (show-basic (tt "       |  <num>"))
    (show-basic (tt "       |  <prim>"))
    (show-basic (tt "       |  <bool>"))
    (blank 15)
    (show-core (tt "       |  (<exp> <exp> ...)"))
    (show-core (tt "       |  (λ (<params>) <body>)"))
    (blank 15)  
    (show-basic (tt "       |  (if <exp> <exp> <exp>)"))
    (show-sugar (tt "       |  (cond (<exp> <exp>) ...)"))
    (blank 15)
    (show-sugar (tt "       |  (let ((<var> <exp>)  ...) <body>)"))
    (show-sugar (tt "       |  (let* ((<var> <exp>) ...) <body>)"))
    (show-sugar (tt "       |  (letrec ((<var> <exp>) ...) <body>)"))
    (blank 15)
    (show-sugar (tt "       |  (quote <sexp>)"))
    (show-sugar (tt "       |  (quasiquote <qq-sexp>)"))
    (blank 15)
    (show-global (tt "       |  (begin <body>)"))
    (show-global (tt "       |  (set! <var> <exp>)"))
    )))

;(subset-scheme-slide id id id id)
;(subset-scheme-slide id id id ghost)
;(subset-scheme-slide id id ghost ghost)
;(subset-scheme-slide id ghost ghost ghost)




(slide 
 (large-text "Um passeio pelo Racket"))



(slide
 #:title "Números"
 (item "Inteiros: " (code 1) "," (code -3))
 
 (item "Racionais: " (code-reduce (/ 8 10)))
 
 (item "\"Reais\": " (code-reduce (sqrt 2)))
 
 (item "Imaginários: " (code-reduce (sqrt -1)))
 
 (item "Complexos: " (code-reduce (* 1+3i 1-22/7i)))
 
 (item "Precisos: " (code-reduce (fact 18))))

(slide
 #:title "Booleanos"
 (item "Literais booleanos " (code #t) ", " (code #f))
 (item "Só " (code #f) " é falso")
 (item "Qualquer outra coisa é verdadeira")
 )




(slide
 #:title "Listas"
 (item (code cons) ", " (code car) ", " (code cdr) ", " (code '())
       ", " (code pair?))
 'next
 (item (code-reduce (cons 1 '())))
 'next
 (item (code-reduce (cons 3 (cons 1 '()))))
 'next
 (item (code-reduce '(3 1)))
 'next
 (item (code-reduce (car (cons 3 (cons 1 '())))))
 'next
 (item (code-reduce (cdr (cons 3 (cons 1 '())))))
 'next
 (item (code-reduce (pair? (cons 3 4))))
 'next
 (item (code-reduce (null? '())))
 )


(slide
 #:title "Quoted S-Expressions"
 (item (code '<s-exp>) (tt " == ") (tt "(quote <s-exp>)"))
 (item (code-reduce '<id>))
 
 (item (code-reduce '3))
 (item (code-reduce 'a))
 (item (code-reduce '(1 2 3)))
 (item (code-reduce (cdr '(1 2 3))))
 (item (code-reduce '(f (g x))))
 (item (code-reduce (car '(f (g x)))))
 
 (item (code-reduce '())))



(slide
 #:title "Primitivas"
 (item "Aritméticas: " (code +) ", " (code modulo) ", " (code quotient))
 (item "Tipos: " (code null?) ", " (code pair?) ", " (code number?) ", " (code integer?))
 (item "Listas: " (code list) comma (code cons) ", " (code car)", " (code cdr))
 (item "Igualdade: " (code =)", " (code eq?)", " (code eqv?) comma (code equal?))
 (item "Ex:" (code-reduce (gcd 248 184)))
 (item "Ex:" (code-reduce (cons 3 (cons 4 '()))))
 )






(slide 
 #:title (code let)
 
 (item (para #:width 200 #:align 'left 
             (tt "<let-exp> ::= ") 
             (tt " (let ((<var> <exp>) ...) ")
             (tt "  <body>)")))
 (item (code let) " atribui valores a nomes de variáveis")
 'next
 (item (code-reduce (let ((π   3.14)
                          (two 2))
                      (* two π))))
 'next
 (item (code (let ((π   3.14)
                   (2*π (* 2 π)))
               2*π)) (colorize (tt " =/=> ") "red") (code 6.28)))




(slide 
 #:title (code let*)
 
 (item (para #:width 200 #:align 'left 
             (tt "<let*-exp> ::= ") 
             (tt " (let* ((<var> <exp>) ...) ")
             (tt "  <body>)")))
 (item (code let*) " atribui valores a nomes de variáveis em sequência")
 'next
 (item (code-reduce (let* ((π   3.14)
                           (π   3.14159))
                      (* 2 π))))
 'next
 (item (code-reduce (let* ((π   3.14)
                           (2*π (* 2 π)))
                      2*π))))




(slide
 #:title "Variáveis"
 
 (item "Qualquer símbolo é permitido")
 
 (item "Grosseiramente: " (tt "[^()'`,]+"))
 
 (item "Ex:" (code foo))
 
 (item "Ex:" (code foo-bar))
 
 (item "Ex:" (code π) comma (code Δ) comma (code ∞) comma (code Σ) comma (code ⊑) comma (code ∅) comma (code →))
 
 (item "Ex:" (code pair?))
 
 (item "Ex:" (code call-with-current-continuation))
 
 (item "Ex:" (code-reduce (let ((let 3)) let))))




(slide
 #:title "λ"
 (item "λ produz funções anônimas")
 (item (tt "<lambda> ::= (λ (<var> ...) <body>)"))
 ; (item (tt "<params> ::= <var> ..."))
 (item (code-reduce ((λ (x) (* x x)) 3)))
 (item (code (let ((λ (λ (λ) (λ λ)))) (λ λ)))))




(slide
 #:title "Escopo léxico"
 
 (t "O que faz o código abaixo?")
 
 (blank)
 
 (code
  (let ((x 20))
    (let* ((x 3)
           (f (λ () x))
           (x 10))
      (f)))))



(slide
 #:title "Escopo léxico"
 (item "λ-terms capturam vínculos no escopo")
 (item "WYSIWYG para funções anônimas"))





(slide
 #:title "Exemplo obrigatório com derivadas"
 
 (code
  (define ε 0.001)
  
  (define (D f) (λ (x) (/ (- (f (+ x ε)) (f x))
                          ε)))
  
  (define (g x) (* x x))
  
  ((D g) 0) ≈ 0
  
  ((D g) 2) ≈ 4))




(slide
 #:title "Condicionais"
 (item (tt "<if-exp> ::= (if <exp> <exp> <exp>)"))
 (item (tt "<cond-exp> ::= ") (tt " (cond (<exp> <exp>) ...)"))
 (item (code-reduce (if 'a 'b 'c)))
 (item (code-reduce (if #f 'b 'c)))
 (item (code-reduce (cond
                      ((< 3 0) 'negativo)
                      ((= 3 0) 'zero)
                      ((> 3 0) 'positivo)))))



(slide
 #:title "Quasiquotes"
 (item "Sintaxe especial para a construção de literais")
 (item (tt "`<qq-sexp>") (tt " == ") (tt "(quasiquote <qq-sexp>)"))
 (item (tt ",<exp>") (tt " == ") (tt "(unquote <exp>)"))
 (item (code-reduce `(3 + 4 is ,(+ 3 4))))
 (code
  (let ((new-ship (λ (name x y)
                    `(ship (name ,name)
                           (x    ,x)
                           (y    ,y)))))
    (new-ship 'Enterprise 42 1701)))
 'next
 (code => (ship (name Enteprise) 
                (x 42) 
                (y 1701))))


(slide
 #:title "Variáveis mutáveis"
 
 (item (code begin) " sequencia expressões, como " (tt "{}")  " em C")
 
 (code-reduce
  (begin
    1
    2
    3))
 
 (item (code set!) " atribui a uma variável, como " (tt "=") " em C")
 
 (code-reduce
  (let ((π 3.14))
    (begin
      (set! π 3.14159)
      π)))
 
 )


(slide 
 #:title "Casamento de padrões"
 (item "Ferramenta poderosa para trabalhar com dados, incluindo estruturas complexas como árvores, URLs, ...")
 'next
 (code-reduce
  (match '(1 2 3)
    [`(1 ,x 4)  x]
    [`(1 2 ,z)  z]))
 'next
 (code-reduce
  (match '(1 3 5 9)
    [(list 1 (? even?) ...)
     'all-even]
    [(list 1 (? odd?) ...)
     'all-odd])))


(slide
 #:title "Quer mais?"
 logo
 (tt "http://racket-lang.org")
 'next
 (blank 50)
 (colorize (t "Esta apresentação foi escrita em Racket, no DrRacket!") "navy")
 'next
 (hc-append (tt " => ")
            (colorize (t "Show me the code!") "red")))
