#lang slideshow
; Rodolfo Henrique Carvalho
; 2011/05/17

(require slideshow/code)
(require slideshow/face)

(set-page-numbers-visible! #f)

;; Library.

(define (large-text txt)
  (text txt (current-main-font) 62))

(define-syntax code-reduce
  (syntax-rules ()
    ((_ exp) 
     (let ((t exp))
       (hb-append (code exp) (tt " => ") (code (unsyntax t)))))))


(define logo
  (bitmap (build-path (collection-path "icons") "PLT-206-larval.png")))


;; ------- FizzBuzz -------
(define (divisible? m n)
  (if (zero? n)
      #f
      (zero? (remainder m n))))

(define (fizzbuzz n)
  (cond
    [(divisible? n 15) "fizzbuzz"]
    [(divisible? n 3) "fizz"]
    [(divisible? n 5) "buzz"]
    [else n]))
;; ------------------------


;; Slides.

(slide
 (para #:align 'center 
       (large-text "TDD com Racket"))
 (blank 50)
 logo
 (vc-append 0
            (t "Rodolfo Carvalho")
            (blank 50)
            (colorize (bt "Dojo Rio") "red")
            (colorize (t "rodolfocarvalho.net") "blue")))


(slide
 #:title "RackUnit"
 (item "Feito para iniciantes e experientes")
 (item "Flexibilidade similar ao Test::More (Perl)")
 'next
 'alts
 (list (list
        (para #:align 'left
              (code (require rackunit)))
        'alts
        (list (list
               (para #:align 'left
                     (code (check equal?
                                  (fizzbuzz 3) "fizz"
                                  "3 is fizz")
                           #,(blank 20)
                           (check equal?
                                  (fizzbuzz 5) "buzz"
                                  "5 is buzz"))))
              (list
               (para #:align 'left
                     (code (test-case
                            "3 is fizz"
                            (check-equal? (fizzbuzz 3) "fizz"))
                           #,(blank 20)
                           (test-case
                            "5 is buzz"
                            (check-equal? (fizzbuzz 5) "buzz")))))
              (list
               (para #:align 'left
                     (code (test-equal?
                            "3 is fizz"
                            (fizzbuzz 3) "fizz")
                           #,(blank 20)
                           (test-equal?
                            "5 is buzz"
                            (fizzbuzz 5) "buzz"))))))
       (list
        (para #:align 'left
              (code (require rackunit rackunit/gui)
                    #,(blank 20)
                    (define-test-suite
                      FizzBuzz                
                      (test-equal?
                       "3 is fizz"
                       (fizzbuzz 3) "fizz")
                      #,(blank 20)
                      (test-equal?
                       "5 is buzz"
                       (fizzbuzz 5) "buzz"))
                    #,(blank 20)
                    (test/gui FizzBuzz))))))


(slide
 #:title "Mini-retrospectiva"
 (hc-append
  (* 3 gap-size)
  (vc-append (* 3 gap-size)
             (face 'happy "green")
             (t "Gostaram da apresentação?"))
  (vc-append (* 3 gap-size)
             (face 'badly-embarassed "red")
             (t "Como posso melhorar?"))))


(slide
 #:title "Sugestão de problema"
 (parameterize ([current-font-size 20])
   (vc-append
    (large-text "Caixa Eletrônico")
    (blank 30)
    (tt "http://dojopuzzles.com/problemas/exibe/caixa-eletronico/")
    (blank 30)
    (para "Desenvolva um programa que simule a entrega de notas quando um cliente efetuar um saque em um caixa eletrônico. Os requisitos básicos são os seguintes:")
    (item "Entregar o menor número de notas;")
    (item "É possível sacar o valor solicitado com as notas disponíveis;")
    (item "Saldo do cliente infinito;")
    (item "Quantidade de notas infinito (pode-se colocar um valor finito de cédulas para aumentar a dificuldade do problema);")
    (item "Notas disponíveis de R$ 100,00; R$ 50,00; R$ 20,00 e R$ 10,00")
    (blank 15)
    (para "Exemplos:")
    (item "Valor do Saque: R$ 30,00 – Resultado Esperado: Entregar 1 nota de R$20,00 e 1 nota de R$ 10,00.")
    (item "Valor do Saque: R$ 80,00 – Resultado Esperado: Entregar 1 nota de R$50,00 1 nota de R$ 20,00 e 1 nota de R$ 10,00."))))


(slide
 #:title "Você pode precisar..."
 (frame
  (inset 
   (vc-append 20
              (tt "check, check-equal?, test-case, test-equal?")
              (tt "define, let*, set!, for, if, cond")
              (tt "=, eq?, eqv?, equal?")
              (tt "λ, list, cons, car, cdr"))
   10))
 (blank)
 (t "Qualquer dúvida podemos buscar no \"HelpDesk\""))
