#lang racket/base
;; Example of sort algorithms

(require racket/list
         racket/match
         rackunit
         "demo/list-merge.rkt")

(define (pat-sort lst)
  (match lst
    ['() '()]
    [(list (? number? n)) (list n)]
    [(list (? number? n) (? number? m)) (if (n . <= . m)
                                            (list n m)
                                            (list m n))]
    [(list (? number? n) rest ...) (let ([rest/sorted (pat-sort rest)])
                                     (cond
                                       [(n . <= . (first rest/sorted)) (list* n rest/sorted)]
                                       [(n . >= . (last rest/sorted)) (append rest/sorted (list n))]
                                       [else ;(error "I don't know how to sort!" (list n rest/sorted))
                                        ;(displayln (append rest/sorted (list n)))
                                        ; This is very slow, as we end up reordering many times
                                        (pat-sort (append rest/sorted (list n)))]))]))

(define (merge-sort lst)
  (match lst
    ['() '()]
    [(list (? number? n)) (list n)]
    [(list (? number? n) (? number? m)) (if (n . <= . m)
                                            (list n m)
                                            (list m n))]
    [(list (? number? ns) ...) (let-values ([(l r) (split-at ns (truncate (/ (length ns) 2)))])
                                 (list-merge (merge-sort l) (merge-sort r)))]))

;; tests

(define (check-sort algorithm . inputs)
  (for ([input (in-list inputs)])
    (check-equal? (algorithm input) (sort input <))))

(check-sort pat-sort
            '()
            '(1)
            '(1 2 3 4)
            '(4 3 2 1)
            '(2 5 1 3 4)
            '(2 5 1 5 3 4)
            '(80 5 20 17 41 16 77 26 34 0))

(check-sort merge-sort
            '()
            '(1)
            '(1 2 3 4)
            '(4 3 2 1)
            '(2 5 1 3 4)
            '(2 5 1 5 3 4)
            '(80 5 20 17 41 16 77 26 34 0 87 69 52 98 76 39 28 4 49 73 90 17 27 29 46 77 1 55 74 17 35 79 27 52 55 70 38 48 23 64 65 94 2 16 58 51 60 60 97 73 27 21 93 85 41 74 61 88 42 16 41 66 27 36 34 2 84 17 87 89 37 17 95 27 57 6 73 91 79 97 12 44 89 69 14 57 53 77 90 22 79 32 16 12 84 2 39 97 66 90))


(define (match-sequence/1 L)
  (let ([n-lst '()]     ; Used to collect numbers found.
        [s-lst '()])    ; Used to collect strings found.
    (define  (m-test L)
      (match L
        [(list-rest (? number? n) (? string? s)  ... (? m-test))
         (set! n-lst `(,@n-lst ,n))
         (set! s-lst `(,@s-lst ,(car s)))                
         (list (reverse n-lst) (reverse s-lst))]
        ['()
         #t]
        [else
         #f]))
    (m-test L)))

(define (match-sequence/2 L)
  (match L
    [(list* (? number? n) (? string? s) rest)
     ;(printf "~a ~a ~a~n" n s rest)
     (let-values ([(ns ss) (match-sequence/2 rest)])
       (values `(,n ,@ns) `(,s ,@ss)))]
    ['() (values '() '())]))

; split list
(define (match-sequence/3 L)
  (match L
    [(list* (? number? n) (? number? s) rest)
     ;(printf "~a ~a ~a~n" n s rest)
     (let-values ([(ns ss) (match-sequence/3 rest)])
       (values `(,n ,@ns) `(,s ,@ss)))]
    [(list (? number? n)) (values `(,n) '())]
    ['() (values '() '())]))

(define separator (make-string 60 #\-))

(match-sequence/1 '(1 "hello" 2 "world"))
(displayln separator)
(match-sequence/2 '(1 "hello" 2 "world" 3 "banana"))
(displayln separator)
(match-sequence/3 '(1 2 3 4 5 6))
(match-sequence/3 '(1 2 3 4 5 6 7))
(match-sequence/3 '(1 2))
